@@ -3,6 +3,7 @@ Created on 5 avr. 2010
 
 @author: thierry
 '''
+import logging
 
 import os.path
 import mimetypes
@@ -15,7 +16,7 @@ import gdata.geo
 
 from caritang.common import version 
 
-version.getInstance().submitRevision("$Revision: 36 $")
+version.getInstance().submitRevision("$Revision: 73 $")
 
 NOT_YET_IMPLEMENTED = 'not yet implemented'
 
@@ -43,11 +44,43 @@ class DocStorage(object):
         '''
         raise RuntimeError(NOT_YET_IMPLEMENTED)
     
-    
+    def connect(self, captcha=None, response=None):
+        '''
+        connect the storage to the google service.
+        This method generally call programmaticLogin.
+        The caller must handle CaritangCaptchaException
+        '''
+        raise RuntimeError(NOT_YET_IMPLEMENTED)
+        
+    def isConnected(self):
+        '''
+        Test is the storage is connected
+        '''
+        raise RuntimeError(NOT_YET_IMPLEMENTED)
+
 
 class CaritangStorageException(Exception):
     pass
-            
+
+
+class CaritangCaptchaException(Exception):
+    '''
+    Login failure because google required a capcha
+    '''            
+    
+    def __init__(self, token, url, doc_storage=None):
+        self.captcha_token = token
+        self.captcha_url = url
+        self.doc_storage = doc_storage
+        
+    def get_token(self):
+        return self.captcha_token
+    
+    def get_url(self):
+        return self.captcha_url
+    
+    def get_storage(self):
+        return self.doc_storage
     
 class AggregatorStorage(DocStorage):
     '''
@@ -58,6 +91,15 @@ class AggregatorStorage(DocStorage):
     def __init__(self):
         self.storages = []
 
+    def connect(self, captcha=None, response=None):
+        for a_storage in self.storages:
+           a_storage.connect(captcha, response)                 
+
+
+    def isConnected(self):
+        return not False in map(lambda(storage): storage.isConnected(), self.storages)
+
+
     def getName(self):
         return "aggregator"
 
@@ -86,6 +128,8 @@ class Picassa(DocStorage):
     '''
     
     mimetypes.add_type("video/mp4", ".mp4")
+    mimetypes.add_type("image/png", ".png")
+    mimetypes.add_type("image/gif", ".gif")
     gdata.photos.service.SUPPORTED_UPLOAD_TYPES = gdata.photos.service.SUPPORTED_UPLOAD_TYPES + ("mp4",)
     
     def __init__(self, login, password, album=None):
@@ -100,10 +144,26 @@ class Picassa(DocStorage):
         self.gd_client.email = login
         self.gd_client.password = password
         self.gd_client.source = 'bressure.net-caritang-v1'
-        self.gd_client.ProgrammaticLogin()
         
+        self.__is_connected = False                
         self.album = album
-        self.ensure_destination_folder()
+
+
+    def getName(self):
+        return "Picasa"
+
+        
+
+    def isConnected(self):
+        return self.__is_connected
+
+    def connect(self, captcha=None, response=None):
+        try : 
+            self.gd_client.ProgrammaticLogin(captcha, response)
+            self.__is_connected = True
+            self.ensure_destination_folder()
+        except gdata.service.CaptchaRequired:
+            raise CaritangCaptchaException(self.gd_client._GetCaptchaToken(), self.gd_client._GetCaptchaURL(), self)        
 
     def canStore(self, doc):
         type = mimetypes.guess_type(doc.getName(),strict=False)[0]    
@@ -134,16 +194,20 @@ class Picassa(DocStorage):
         doc = docItem.getFile()
         type = mimetypes.guess_type(doc,strict=False)[0]        
         photoName = os.path.basename(doc)
-        if self.album is None:
-            photo = self.gd_client.InsertPhotoSimple('/data/feed/api/user/default/albumid/default', photoName, 
-                                               'Uploaded with Caritang', doc, content_type=type)
-        else :
-            album_url = '/data/feed/api/user/%s/albumid/%s' % (self.login, self.album_entry.gphoto_id.text)
-            photo = self.gd_client.InsertPhotoSimple(album_url, photoName, 
-                            'Uploaded with Caritang', doc, content_type=type)                   
-
-            
-               
+        try:
+            if self.album is None:
+                photo = self.gd_client.InsertPhotoSimple('/data/feed/api/user/default/albumid/default', photoName, 
+                                                         'Uploaded with Caritang', doc, content_type=type)
+            else :
+                album_url = '/data/feed/api/user/%s/albumid/%s' % (self.login, self.album_entry.gphoto_id.text)            
+                photo = self.gd_client.InsertPhotoSimple(album_url, photoName, 
+                                                         'Uploaded with Caritang', doc, content_type=type)                   
+        except gdata.photos.service.GooglePhotosException:
+            logging.exception("Failed to save picture")
+            raise CaritangStorageException() #TODO be more precise
+        except IOError:
+            logging.exception("Failed to save picture")
+            raise CaritangStorageException() #TODO be more precise
             
         
     def __str__(self):        
@@ -165,12 +229,29 @@ class GoogleDocs(DocStorage):
         Note : login must contains the @gmail.com part        
         '''
         self.login = login
-        self.gd_client = gdata.docs.service.DocsService(source='bressure.net-caritang-v1')
-        self.gd_client.ClientLogin(login, password)
+        self.password = password
+        self.gd_client = gdata.docs.service.DocsService(source='bressure.net-caritang-v1')        
+        self.folder = folder        
         
-        self.folder = folder
-        self.ensure_destination_folder()
+        self.__is_connected = False
 
+    def getName(self):
+        return "Google docs"
+    
+    def isConnected(self):
+        return self.__is_connected
+
+
+    def connect(self, captcha=None, response=None):
+        try : 
+            self.gd_client.ClientLogin(self.login, self.password, captcha_token=captcha, captcha_response=response)
+            self.__is_connected = True            
+            self.ensure_destination_folder()
+        except gdata.service.CaptchaRequired:
+            raise CaritangCaptchaException(self.gd_client._GetCaptchaToken(), self.gd_client._GetCaptchaURL(), self)        
+
+
+    
     def canStore(self, doc):
         return False # inhibate google doc 
     
